//---------------------------------------------------------------------------
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#pragma hdrstop

#include "usart.h"
//---------------------------------------------------------------------------

// maximum data block to send at once
#define BLOCK 8192

#define header() \
{ \
	printf("Simple COM port file sender by Stanislav Maslan, s.maslan@seznam.cz,\n"); \
	printf("version V1.00, 2012-05-29.\n"); \
}

#define usage() \
	printf("usage: com_send_file.exe file_path com_number com_baud [-q[uiet]]\n\n");

// ====== open file, get its length ======
int fopenlen(FILE **fr,char *path,int *len)
{
	if(!fr)
		return(1);

	// try to open
	*fr=fopen(path,"rb");
	if(!*fr)
		return(2);

	// get size
	fseek(*fr,0L,SEEK_END);
	*len=(int)ftell(*fr);
	fseek(*fr,0L,SEEK_SET);

	return(0);
}

// ====== main ======
#pragma argsused
int main(int argc, char* argv[])
{
	char *pstr;
	HANDLE com=NULL;

	// check parameters
	if(argc<4)
	{
		header();
		usage();
		printf("Error - not enough parameters!\n\n");
		return(1);
	}

	// quiet?
	int quiet=0;
	if(argc>=5 && (strcmpi(argv[4],"-q")==0 || strcmpi(argv[4],"-quiet")==0))
		quiet=1;

	// print header
	if(!quiet)
		header();

	// get com number
	int comn=(int)strtol(argv[2],&pstr,10);
	if(pstr==argv[2] || comn<1)
	{
		// invalid number
		header();
		usage();
		printf("Error - invalid COM port number!\n\n");
		return(1);
	}

	// get baudrate
	int baud=(int)strtol(argv[3],&pstr,10);
	if(pstr==argv[3] || baud<300)
	{
		// invalid number
		header();
		usage();
		printf("Error - invalid baud rate!\n\n");
		return(1);
	}

	// open COM port
	if(COMopen(&com,comn))
	{
		// opening failed
		header();
		usage();
		printf("Error - opening COM port failed!\n\n");
		return(1);
	}

	// set buffers
	if(COMsetBuf(com,256,BLOCK))
	{
		// config failed
		header();
		usage();
		printf("Error - setting COM buffer size failed!\n\n");
		COMclose(&com);
		return(1);
	}

	// set COM format
	if(COMsetFormat(com,baud,ONE,NO,8,0,0))
	{
		// config failed
		header();
		usage();
		printf("Error - setting COM format failed!\n\n");
		COMclose(&com);
		return(1);
	}

	// set timeouts
	if(COMsetTimeouts(com,0,0,1,1,100))
	{
		// config failed
		header();
		usage();
		printf("Error - setting COM timeouts failed!\n\n");
		COMclose(&com);
		return(1);
	}

	// open source file
	FILE *fr;
	int flen;
	if(fopenlen(&fr,argv[1],&flen))
	{
		// failed
		header();
		usage();
		printf("Error - can't open source file!\n\n");
		COMclose(&com);
		return(1);
	}


	// === send data ===
	int read,towr;
	int fpos=0;
	do{
		unsigned char data[BLOCK];

		// unsent bytes count
		int txq;
		COMtxLen(com,&txq);

		// free space in tx buffer
		towr=BLOCK-txq;

		// read block
		read=fread((void*)data,1,towr,fr);

		// write data into buffer
		if(read && COMtxData(com,data,read)<0)
		{
			// failed
			header();
			usage();
			printf("Error - can't write into COM tx buffer!\n\n");
			fclose(fr);
			COMclose(&com);
			return(1);
		}

		// position
		fpos+=read;

		// print status
		if(!quiet)
			printf("progress: %3d%%\r",(int)(100LL*(long long)fpos/(long long)flen));

		// sleep
		Sleep(5);
	}while(read==towr);

	// === wait for tx completition ===
	int txq;
	do{
		// unset data
		COMtxLen(com,&txq);

		// sleep
		Sleep(5);
	}while(txq);

	// done
	if(!quiet)
		printf("progress: done!\n\n");

	// close com port
	COMclose(&com);

	// close file
	fclose(fr);

	return(0);
}
//---------------------------------------------------------------------------
