//---------------------------------------------------------------------------
#include <windows.h>
#include <stdio.h>
#pragma hdrstop

#include "usart.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

//===========================================================================
// Check for port existance
//===========================================================================
int COMcheck(int comn)
{
	HANDLE comp;
	OSVERSIONINFO osi={sizeof(OSVERSIONINFO)};
	char coms[16];

	// invalid port number
	if(comn<1 || comn>256)
		return(1);

	// generate COM port name
	GetVersionEx(&osi);
	if(osi.dwMajorVersion>=5)
		// version>=5 (W2000, XP, ...)
		sprintf(coms,"\\\\.\\COM%d",comn);
	else
		// version<5 (W95, W98, ...)
		sprintf(coms,"COM%d",comn);

	// try to open
	comp=CreateFile(coms,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	if(comp==INVALID_HANDLE_VALUE)
	{
		// failed
		return(1);
	}

	// succeeded
	CloseHandle(comp);
	return(0);
}

//===========================================================================
// Open specified COM port
//===========================================================================
int COMopen(HANDLE *cporth,int comn)
{
	char coms[16];
	OSVERSIONINFO osi={sizeof(OSVERSIONINFO)};

	// missing parameter
	if(cporth==NULL)
		return(1);

	// invalid port number
	if(comn<1 || comn>256)
		return(2);

	// generate COM port name
	GetVersionEx(&osi);
	if(osi.dwMajorVersion>=5)
		// version>=5 (W2000, XP, ...)
		sprintf(coms,"\\\\.\\COM%d",comn);
	else
		// version<5 (W95, W98, ...)
		sprintf(coms,"COM%d",comn);

	// try to open
	*cporth=CreateFile(coms,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
	if(*cporth==INVALID_HANDLE_VALUE)
	{
		// failed
		*cporth=NULL;
		return(3);
	}

	return(0);
}

//===========================================================================
// Close COM port
//===========================================================================
int COMclose(HANDLE *cporth)
{
	if(cporth==NULL)
		return(1);

	//
	PurgeComm(*cporth,PURGE_TXCLEAR|PURGE_TXABORT);

	// close port
	CloseHandle(*cporth);
	*cporth=NULL;

	return(0);
}


//===========================================================================
// Set recomended COM buffers sizes
//===========================================================================
int COMsetBuf(HANDLE cporth,int inlen,int outlen)
{
	return(!SetupComm(cporth,inlen,outlen));
}


//===========================================================================
// Set COM parameters
//===========================================================================
int COMsetFormat(HANDLE cporth,int baud,EStpb stp,EParf par,int bsz,int rts,int cts)
{
	DCB dcb;

	if(baud<1 || baud>3000000)
		return(1);

	if(bsz<5 || bsz>8)
		return(4);

	// get current settings
	if(!GetCommState(cporth,&dcb))
		return(0x100);

	dcb.fBinary=true;
	dcb.fAbortOnError=false;
	dcb.fNull=false;

	dcb.BaudRate=baud;
	switch(stp)
	{
		case ONE: dcb.StopBits=ONESTOPBIT; break;
		case ONE5: dcb.StopBits=ONE5STOPBITS; break;
		case TWO: dcb.StopBits=TWOSTOPBITS; break;
		default: return(2);
	}
	switch(par)
	{
		case NO: dcb.Parity=NOPARITY; break;
		case ODD: dcb.Parity=ODDPARITY; break;
		case EVEN: dcb.Parity=EVENPARITY; break;
		case SPACE: dcb.Parity=SPACEPARITY; break;
		case MARK: dcb.Parity=MARKPARITY; break;
		default: return(3);
	}
	dcb.ByteSize=bsz;

	dcb.fOutxCtsFlow=(cts==1);
	dcb.fOutxDsrFlow=(cts==2);
	dcb.fRtsControl=(rts==1)?RTS_CONTROL_HANDSHAKE:RTS_CONTROL_ENABLE;
	dcb.fDtrControl=(rts==2)?DTR_CONTROL_HANDSHAKE:DTR_CONTROL_ENABLE;

	// store changed
	if(!SetCommState(cporth,&dcb))
		return(0x101);



	return(0);
}

//===========================================================================
// Set timeouts [ms]
//===========================================================================
int COMsetTimeouts(HANDLE cporth, int rxinterval, int rxtime, int rxmult, int txtime, int txmult)
{
	COMMTIMEOUTS timeouts;
	timeouts.ReadIntervalTimeout=rxinterval;
	timeouts.ReadTotalTimeoutMultiplier=rxmult;
	timeouts.ReadTotalTimeoutConstant=rxtime;
	timeouts.WriteTotalTimeoutMultiplier=txmult;
	timeouts.WriteTotalTimeoutConstant=txtime;
	return(!SetCommTimeouts(cporth,&timeouts));
}

//===========================================================================
// Get unread data count [Bytes]
//===========================================================================
int COMrxLen(HANDLE cporth,int *tord)
{
	DWORD dw;
	COMSTAT cms;

	if(tord==NULL)
		return(1);

	if(!ClearCommError(cporth,&dw,&cms))
		return(2);

	*tord=cms.cbInQue;
	return(0);
}

//===========================================================================
// Get unsent data count [Bytes]
//===========================================================================
int COMtxLen(HANDLE cporth,int *towr)
{
	DWORD dw;
	COMSTAT cms;

	if(towr==NULL)
		return(1);

	if(!ClearCommError(cporth,&dw,&cms))
		return(2);

	*towr=cms.cbOutQue;
	return(0);
}

//===========================================================================
// Flush input buffer data
//===========================================================================
int COMflushInput(HANDLE cporth)
{
	return(!PurgeComm(cporth,PURGE_RXCLEAR));
}

//===========================================================================
// Flush output buffer data
//===========================================================================
int COMflushOutput(HANDLE cporth)
{
	return(!PurgeComm(cporth,PURGE_TXCLEAR|PURGE_TXABORT));
}


//===========================================================================
// Send byte (returns written bytes count)
//===========================================================================
int COMtxByte(HANDLE cporth,unsigned char byte)
{
	DWORD written;

	if(!WriteFile(cporth,&byte,1,&written,NULL))
		return(-1);
	return((int)written);
}

//===========================================================================
// Get byte (returns recieved bytes count)
//===========================================================================
int COMrxByte(HANDLE cporth,unsigned char *byte)
{
	DWORD read;
	if(!ReadFile(cporth,byte,1,&read,NULL))
		return(-1);
	return((int)read);
}

//===========================================================================
// Send data (returns written bytes count)
//===========================================================================
int COMtxData(HANDLE cporth,void *data,int len)
{
	DWORD written;
	if(!WriteFile(cporth,data,(DWORD)len,&written,NULL))
		return(-1);
	return((int)written);
}

//===========================================================================
// Get data (returns recieved bytes count)
//===========================================================================
int COMrxData(HANDLE cporth,void *data,int len)
{
	DWORD read;
	OVERLAPPED ovlp={0,0,0,0,NULL};

	if(!ReadFile(cporth,data,(DWORD)len,&read,&ovlp))
		return(-1);

	// ################################### was ; at end of while()???
	while(!HasOverlappedIoCompleted(&ovlp))
		GetOverlappedResult(cporth,&ovlp,&read,false);

	return((int)read);
}