//---------------------------------------------------------------------------
#include <stdio.h>
#include <vcl.h>
#pragma hdrstop

//---------------------------------------------------------------------------

typedef unsigned char uchar;
typedef unsigned short uint16;

// ====== convert 24bit BIT to 5-6-5 RAW ======
int bmp24_to_raw16(char *path, int rot)
{
	// try to create bitmap
	Graphics::TBitmap *bmp;
	try{
		bmp=new Graphics::TBitmap;
	}catch(Exception &exc){
		// failed
		printf("Error - creating TBitmap class failed with exception:\n\"%s\"\n",exc.Message.c_str());
		return(1);
	}

	// try to open source bitmap
	try{
		bmp->LoadFromFile(AnsiString(path));
	}catch(Exception &exc){
		// failed
		printf("Error - opening bitmap failed with exception:\n\"%s\"\n",exc.Message.c_str());
		return(1);
	}

	// check its format
	if(bmp->PixelFormat!=pf24bit)
	{
		// invalid format
		printf("Error - invalid format, image must be 24-bit raster!\n");
		delete(bmp);
		return(1);
	}

	// get resolution
	int xsize=bmp->Width;
	int ysize=bmp->Height;

	// generate destination path
	char fname[260];
	strcpy(fname,path);
	char *str;
	if((str=strrchr(fname,'.'))!=NULL)
		strcpy(str,".raw");
	else
		strcat(fname,".raw");

	// create output file
	FILE *fw=fopen(fname,"wb");
	if(!fw)
	{
		// creating new file failed
		printf("Error - can't create destination file!\n");
		return(1);
	}

	// source path
	printf("source: %s\n",path);

	// convert image
	int i,j;
	for(j=0;j<ysize;j++)
	{
		uchar *pl=(uchar*)bmp->ScanLine[rot?(ysize-j-1):j] + (rot?(xsize*3):0);

		for(i=0;i<xsize;i++)
		{
			// get RGB
			uint16 rr;
			uint16 gg;
			uint16 bb;
			if(rot)
			{
				rr=*--pl;
				gg=*--pl;
				bb=*--pl;
			}
			else
			{
				bb=*pl++;
				gg=*pl++;
				rr=*pl++;
			}

			// build pixel in 5-6-5 format
			uint16 pix=(rr>>3)|((gg<<3)&0x07E0)|((bb<<8)&0xF800);

			// write pixel
			fwrite((void*)&pix,1,2,fw);
		}
	}

	// close file
	fclose(fw);

	// dest path
	printf("dest.:  %s\n",fname);

	return(0);
}

// ====== header ======
#define header() { \
	printf("24-bit BMP to RAW 5-6-5 convertor by Stanislav Maslan, s.maslan@seznam.cz,\n"); \
	printf("version V1.00, 2012-06-02.\n"); \
}

// ====== usage ======
#define usage()	printf("usage: raw_gen.exe bmp_path [-r/rotate]\n\n")

#pragma argsused
int main(int argc, char* argv[])
{
	// print header
	header();

	if(argc<2)
	{
		// missing parameters
		usage();
		printf("Error - missing parameters!\n\n");
		return(1);
	}
	printf("\n");

	// rotate?
	int rot=argc>2 && (strcmpi(argv[2],"-r")==0 || strcmpi(argv[2],"-rotate")==0);

	// convert bmp2raw
	int err=bmp24_to_raw16(argv[1],rot);

	printf("\n");

	// done
	return(err);
}
//---------------------------------------------------------------------------
